import { NextFunction, Request, Response } from "express";
import { ApiError } from "../utils/ApiError";
import { ITodo } from "../schemas/todoSchema";
import { TodoModel } from "../models/todoModel";
import { RabbitMQ } from "../rabbitmq/RabbitMQ";

export class TodoController {
  private todoModel: TodoModel;
  queue: RabbitMQ;

  constructor() {
    this.todoModel = new TodoModel();
    this.queue = new RabbitMQ();
  }

  public getAll = async (req: Request, res: Response, next: NextFunction) => {
    const todos: ITodo[] | ApiError = await this.todoModel.findAll();
    if (todos instanceof ApiError) {
      return next(todos);
    }

    return res.json(todos);
  };

  public getOne = async (req: Request, res: Response, next: NextFunction) => {
    const todo: ITodo | ApiError = await this.todoModel.findOne(req.params.id);
    if (todo instanceof ApiError) {
      return next(todo);
    }
    return res.json(todo);
  };

  public createOne = async (
    req: Request,
    res: Response,
    next: NextFunction
  ) => {
    try {
      const todo: ITodo | ApiError = await this.todoModel.create(req.body);
      if (todo instanceof ApiError) {
        return next(todo);
      }
      await this.queue.create(todo);

      return res.json(todo);
    } catch {
      const err = new ApiError(
        "Internal server error",
        500,
        "Internal Server Error"
      );
      next(err);
    }
  };

  public updateOne = async (
    req: Request,
    res: Response,
    next: NextFunction
  ) => {
    const todo: ITodo | ApiError = await this.todoModel.update(
      req.body,
      req.params.id
    );
    if (todo instanceof ApiError) {
      return next(todo);
    }

    await this.queue.create(todo);

    return res.json(todo);
  };

  public deleteOne = async (
    req: Request,
    res: Response,
    next: NextFunction
  ) => {
    const { id } = req.params;
    const todo: boolean | ApiError = await this.todoModel.remove(id);
    if (!todo) {
      const error = new ApiError("Todo not found", 404, "Not Found");
      return next(error);
    }
    return res.json(todo);
  };

  public removeAll = async (
    req: Request,
    res: Response,
    next: NextFunction
  ) => {
    const todos: boolean | ApiError = await this.todoModel.removeAll();
    return res.json(todos);
  };
}
